__title__ = "aiolancium"
__package__ = "aiolancium"
__summary__ = "Simplistic python asyncio REST client for the Lancium Compute REST API"
__url__ = "https://github.com/giffels/aiolancium"

__version__ = "0.2.2"
__author__ = "Manuel Giffels"
__email__ = "giffels@gmail.com"
__copyright__ = "2022 %s and Contributors" % __author__
__keywords__ = ["asyncio", "lancium", "compute client"]
